<?php

namespace ChangeCommerce\Core\Observer;

use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\HTTP\Adapter\Curl;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\MessageQueue\Publisher;
use ChangeCommerce\Core\Model\OrderWebhookFailureFactory;
use ChangeCommerce\Core\Logger\Logger;
use ChangeCommerce\Core\Helpers\OrderObjHelper;
use ChangeCommerce\Core\Model\Config\Data as Config;

abstract class AbstractOrderObserver implements ObserverInterface
{
    // to be populated by implementation
    protected $uri;
    protected $verb;
    protected $hookName;
    protected $topic;

    abstract protected function getOrderId(Observer $observer);

    // from constructor
    protected $_curlAdapter;
    protected $_jsonHelper;
    protected $_orderObjHelper;
    protected $_publisher;
    protected $_logger;
    protected $_failureFactory;
    protected $_config;

    public function __construct(
        Curl $curlAdapter,
        Json $jsonHelper,
        Publisher $publisher,
        Logger $logger,
        OrderWebhookFailureFactory $failureFactory,
        OrderObjHelper $orderObjHelper,
        Config $config,
        $uri, $verb, $hookName, $topic)
    {
        $this->_curlAdapter = $curlAdapter;
        $this->_jsonHelper = $jsonHelper;
        $this->_orderObjHelper = $orderObjHelper;
        $this->_publisher = $publisher;
        $this->_logger = $logger;
        $this->_failureFactory = $failureFactory;
        $this->_config = $config;

        $this->uri = $uri;
        $this->verb = $verb;
        $this->hookName = $hookName;
        $this->topic = $topic;
    }

    public function execute(Observer $observer) {
        try {

            if (!$this->_config->isEnabled()) {
                $this->_logger->info("Store not enabled, skipping execution");
                return;
            }

            $this->_logger->info("Publishing event for order {$this->getOrderId($observer)}");
            $this->_publisher->publish($this->topic, $this->getOrderId($observer));
        } catch (\Exception $e) {
            $this->_logger->error("ERROR: Failed to publish event for order {$e->getMessage()}");
        }
    }

    public function makeWebhookCall($id) 
    {
        $headers = null;
        $bodyJson = null;
        $response = null;

        try {
            $orderObj = $this->_orderObjHelper->getOrderObjById($id);
            $bodyJson = $this->_jsonHelper->serialize($orderObj);
            $headers = ["Content-Type: application/json", 
            "x-store-id: {$this->_config->getStoreId()}",
            "x-store-api-key: {$this->_config->getSecret()}"];
        } catch (\Exception $e) {
            $this->_logger->error("ERROR: Unable to get order information for ChangeCommerce Webhook for order {$id} {$e->getMessage()}");
            $failure->saveFailure(0, $this->hookName, $bodyJson, $this->uri, $this->verb, $this->_jsonHelper->serialize($headers), null, null, $e->getMessage(), $id);
            return;
        }

        try {
            $this->_logger->info("Sending http webhook to {$this->verb} {$this->uri}");
            $this->_curlAdapter->write($this->verb, $this->uri, '1.1', $headers, $bodyJson);
            $response = $this->_curlAdapter->read();
            $this->_curlAdapter->close();
        } catch (\Exception $e) {
            $this->_logger->error("ERROR: Failed to send webhook for order {$id} {$e->getMessage()}");
            $failure->saveFailure(1, $this->hookName, $bodyJson, $this->uri, $this->verb, $this->_jsonHelper->serialize($headers), null, null, 'Failed to send webhook for order: ' . $e->getMessage(), $id);
            throw $e;
        }

        $responseStatus = \Zend_Http_Response::extractCode($response);
        $this->_logger->info("Response status: {$responseStatus}");

        if ($responseStatus >= 200 && $responseStatus < 300) {
            return true;
        }

        $this->_logger->error("ERROR: Response failure from sending webhook for order {$id} {$response}");
        $failure = $this->_failureFactory->create();
        $failure->saveFailure(2, $this->hookName, $bodyJson, $this->uri, 
            $this->verb, $this->_jsonHelper->serialize($headers), $response, $responseStatus, 'Response failure from sending webhook for order', $id);
    }
}